#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <termios.h>
#include <unistd.h>

#define PERMS 0666

void	fatal(char *);

main(int argc, char *argv[])
{
	void	copy_one_file(char *, char *);
	void	copy_many_file(char **, int);
	struct	stat	file_stat;


	if(argc < 3)
		{
		puts(" : a.out source object");
		puts(" : a.out source1 source2 ... directory");
		exit(0);
		}


	if(stat(argv[1], &file_stat) < 0)
		fatal("stat");

	/* a.out source object : source is directory */
	/* error */
	if(argc == 3 && S_ISDIR(file_stat.st_mode & S_IFMT))
		{
		printf("%s is directory\n", argv[1]);
		exit(1);
		}


	/* a.out source object : source is not directory */
	/* a.out * object : object is directory */
	if(!access(argv[argc - 1], F_OK))
	if(stat(argv[argc - 1], &file_stat) < 0)
		fatal("stat");
	if(argc == 3)
		copy_one_file(argv[1], argv[2]);
	else	if(argc > 3 && S_ISDIR(file_stat.st_mode & S_IFMT))
		copy_many_file(++argv, argc - 1);
	else	printf("%s is not directory\n", argv[argc - 1]);

}

void
copy_one_file(char *source, char *object)
{
	int	source_fd, object_fd;
	int	read_number;
	char	answer[32];
	char	buff[BUFSIZ];
	char	full_object[BUFSIZ];
	struct	stat	source_file_stat, object_file_stat;


	if((source_fd = open(source, O_RDONLY)) < 0)
		fatal("read open");
	if((object_fd = open(object, O_WRONLY | O_CREAT | O_EXCL, PERMS)) < 0)
		{
		if(stat(object, &object_file_stat) < 0)
			fatal("object stat");
		if(S_ISDIR(object_file_stat.st_mode & S_IFMT))
			{
			sprintf(full_object, "%s/%s", object, source);
			close(source_fd);
			copy_one_file(source, full_object);
			return;
			}

		printf("cp: overwrite '%s' ? ", object);
		fgets(answer, sizeof(answer), stdin);
		if(strncmp(answer, "y\n", 2) && strncmp(answer, "Y\n", 2))
			return;
		}


	if(stat(source, &source_file_stat) < 0)
		fatal("source stat");


	if(S_ISDIR(source_file_stat.st_mode & S_IFMT))
		{
		printf("%s : This is directory\n", source);
		return;
		}

	strcpy(full_object, object);
	if((object_fd = open(full_object, O_WRONLY | O_CREAT | O_TRUNC, PERMS)) < 0)
		fatal("open");

	while((read_number = read(source_fd, buff, BUFSIZ)) > 0)
		if(write(object_fd, buff, read_number) < read_number)
			fatal("write");

	close(source_fd);
	close(object_fd);
}

void
copy_many_file(char **argument, int count)
{
	int	i;
	char	buff[BUFSIZ];

	strcpy(buff, argument[count - 1]);

	for(i = 0;i < count - 1;i++)
		copy_one_file(*argument++, buff);

}

void
fatal(char *error_name)
{
	perror(error_name);
	exit(1);
}
